## ------------------------------------------
## CLUSTER parameter groups
## ------------------------------------------

## There are 3 parameter group scenarios (by priority)...
## 1) user-supplied parameter group name. If users supplies this, we use it.
## 2) user-supplied parameter values. If user supplies these, we'll build a parameter group as long
##    as they didn't supply a parameter group name.
## 3) if neither 1 or 2 are provided, the respective RDS resource uses the AWS default parameter group for that engine.

## Note: there must be separate evaluations for both "cluster" and "instance" Parameter Groups

locals {
  enable_custom_aws_rds_cluster_parameter_group = var.cluster_parameter_group_name == null && var.cluster_parameter_mapping != null ? true : false

  family = "aurora-postgresql${split(".", var.engine_version)[0]}"

  aws_rds_cluster_default_params = [
    {
      name         = "rds.force_autovacuum_logging_level"
      value        = "log"
      apply_method = "immediate"
    },
    {
      name         = "max_connections"
      value        = var.max_connections
      apply_method = "pending-reboot"
    },
    {
      name         = "log_autovacuum_min_duration"
      value        = "100"
      apply_method = "immediate"
    },
    {
      name         = "autovacuum_vacuum_scale_factor"
      value        = "0.02"
      apply_method = "immediate"
    },
    {
      name         = "rds.force_ssl"
      value        = "1"
      apply_method = "immediate"
    }
  ]

  ## we need to merge the module-defined parameters with the user-supplied ones. Otherwise,
  ## they will ping-pong between the two values (if different) ...

  ## project values into maps based on the 'name' key
  cluster_default_param_map = { for i in local.aws_rds_cluster_default_params : i.name => i }
  cluster_user_param_map    = var.cluster_parameter_mapping != null ? { for i in var.cluster_parameter_mapping : i.name => i } : {}
  ## merge both maps using the user-supplied map as the overload
  merged_cluster_params_map = merge(
    local.cluster_default_param_map,
    local.cluster_user_param_map,
  )
  ## grab the values of the merged map as the final set of parameters
  cluster_params = toset(values(local.merged_cluster_params_map))
}

resource "aws_rds_cluster_parameter_group" "aurora_cluster_parameter_group" {
  count       = local.enable_custom_aws_rds_cluster_parameter_group ? 1 : 0
  name        = "${local.db_cluster_name}-cluster-pg"
  family      = local.family
  description = "${local.db_cluster_name} psc cluster parameter group"
  tags        = var.tags

  dynamic "parameter" {
    for_each = local.cluster_params
    content {
      name         = parameter.value.name
      value        = parameter.value.value
      apply_method = lookup(parameter.value, "apply_method", null)
    }
  }

  lifecycle {
    create_before_destroy = true
  }
}

## changing db_cluster_parameter_group_name isn't immediate. So, this is a hack to allow for a change
## and modify before attempting to destroy the custom parameter group
resource "time_sleep" "aws_rds_cluster_parameter_group" {
  depends_on = [aws_rds_cluster_parameter_group.aurora_cluster_parameter_group]

  destroy_duration = "60s"
}

## ------------------------------------------
## Cluster Instance parameter groups
## ------------------------------------------

locals {
  enable_custom_aws_db_parameter_group = var.instance_parameter_group_name == null && var.instance_parameter_mapping != null ? true : false

  aws_db_default_params = [
    {
      name         = "random_page_cost"
      value        = "1"
      apply_method = "pending-reboot"
    },
    {
      name         = "seq_page_cost"
      value        = "1"
      apply_method = "pending-reboot"
    },
    {
      name         = "maintenance_work_mem"
      value        = "{DBInstanceClassMemory/16384}"
      apply_method = "pending-reboot"
    },
    {
      name         = "max_connections"
      value        = var.max_connections
      apply_method = "pending-reboot"
    },
    {
      name         = "max_files_per_process"
      value        = "100000"
      apply_method = "pending-reboot"
    },
    {
      name         = "pg_stat_statements.track"
      value        = "ALL"
      apply_method = "pending-reboot"
    },
    {
      name         = "shared_preload_libraries"
      value        = "pg_stat_statements,pgaudit"
      apply_method = "pending-reboot"
    },
    {
      name         = "track_activity_query_size"
      value        = "2048"
      apply_method = "pending-reboot"
    },
    {
      name         = "log_statement"
      value        = "none"
      apply_method = "immediate"
    },
    {
      name         = "pgaudit.role"
      value        = "rds_pgaudit"
      apply_method = "pending-reboot"
    }
  ]

  ## we need to merge the module-defined parameters with the user-supplied ones. Otherwise,
  ## they will ping-pong between the two values (if different) ...

  ## project values into maps based on the 'name' key
  db_default_params = { for i in local.aws_db_default_params : i.name => i }
  db_user_param_map = var.instance_parameter_mapping != null ? { for i in var.instance_parameter_mapping : i.name => i } : {}
  ## merge both maps using the user-supplied map as the overload
  merged_db_params_map = merge(
    local.db_default_params,
    local.db_user_param_map,
  )
  ## grab the values of the merged map as the final set of parameters
  db_params = toset(values(local.merged_db_params_map))
}

resource "aws_db_parameter_group" "aurora_db_parameter_group" {
  count       = local.enable_custom_aws_db_parameter_group ? 1 : 0
  name        = "${local.db_cluster_name}-db-pg"
  family      = local.family
  description = "${local.db_cluster_name} psc db parameter group"
  tags        = var.tags

  # AMAZON LIES
  # parameter {
  #   name         = "effective_io_concurrency"
  #   value        = "4"
  #   apply_method = "pending-reboot"
  # }

  dynamic "parameter" {
    for_each = local.db_params
    content {
      name         = parameter.value.name
      value        = parameter.value.value
      apply_method = lookup(parameter.value, "apply_method", null)
    }
  }

  lifecycle {
    create_before_destroy = true
  }
}

## changing db_parameter_group_name isn't immediate. So, this is a hack to allow
## time for a change to complete before attempting to destroy the custom parameter group
resource "time_sleep" "aws_db_parameter_group" {
  depends_on = [aws_db_parameter_group.aurora_db_parameter_group]

  destroy_duration = "60s"
}
