variable "allow_major_version_upgrade" {
  description = "Allow major engine version upgrades when changing engine versions"
  type        = bool
  default     = true
}

variable "apply_immediately" {
  description = <<-EOT
    (Optional) Specifies whether any database modifications
    are applied immediately, or during the next maintenance
    window. Default is false. See Amazon RDS Documentation
    for more information.
  EOT
  type        = bool
  default     = false
}

variable "db_cluster_name" {
  description = "Custom cluster name"
  type        = string
  default     = ""
}

variable "port" {
  description = "(Optional) The port on which the DB accepts connections"
  type        = number
}

variable "admin_username" {
  description = <<-EOT
    (Required unless a snapshot_identifier or replication_source_identifier is provided
    or unless a global_cluster_identifier is provided when the cluster is the "secondary"
    cluster of a global database) Username for the master DB user. Please refer to the
    RDS Naming Constraints. This argument does not support in-place updates and cannot
    be changed during a restore from snapshot.
  EOT
  type        = string
}

variable "password" {
  description = <<-EOT
    (Required unless a snapshot_identifier or replication_source_identifier is provided
    or unless a global_cluster_identifier is provided when the cluster is the "secondary"
    cluster of a global database) Password for the master DB user. Note that this may
    show up in logs, and it will be stored in the state file. Please refer to the RDS
    Naming Constraints
  EOT
  type        = string
}

variable "engine_version" {
  description = "RDS Aurora PostgreSQL cluster engine version"
  type        = string
  default     = "15.5"
}

variable "enable_replicas" {
  type    = bool
  default = false
}

variable "enable_failover" {
  type    = bool
  default = true
}

variable "replica_count" {
  type    = number
  default = 1
}

variable "instance_type" {
  description = <<-EOT
    (Required) The instance class to use. For details on CPU and memory, see Scaling
    Aurora DB Instances. Aurora uses db.* instance classes/types. Please see AWS
    Documentation for currently available instance classes and complete details.
  EOT
  type        = string
}

variable "subnet_ids" {
  type    = list(string)
  default = []
}

variable "security_group_ids" {
  type    = list(string)
  default = []
}

variable "backup_retention_period" {
  type    = number
  default = 7
}

variable "backup_window" {
  type    = string
  default = "04:35-05:05"
}

variable "replication_source_identifier" {
  type    = string
  default = ""
}

variable "database_name" {
  description = <<-EOT
    (Optional) Name for an automatically created database on cluster creation.
    There are different naming restrictions per database engine.
    See: https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_Limits.html#RDS_Limits.Constraints
  EOT
  type        = string
}

variable "enable_performance_insights" {
  description = "Enables Performance Insights RDS dashboard."
  type        = bool
  default     = false
}

variable "deletion_protection" {
  description = "Enables deletion protection for databse instance."
  type        = bool
  default     = false
}

variable "aurora_cluster_role_arns" {
  description = "IAM Role arns for aurora cluster to assume"
  type        = list(string)
  default     = []
}

variable "tags" {
  description = "Tags for Aurora"
  type        = map(string)
}

## --------------------------------------------------------
## parameter_groups.tf
## --------------------------------------------------------

variable "cluster_parameter_group_name" {
  description = "Name of parameter group to use for the 'aws_rds_cluster' resource."
  type        = string
  default     = null
}

variable "max_connections" {
  type    = string
  default = "1024"
}

variable "cluster_parameter_mapping" {
  description = "Map of cluster params to configure."
  type        = list(map(string))
  default     = null
}

variable "instance_parameter_group_name" {
  description = "Name of parameter group to use for the 'aws_rds_cluster_instance' resource."
  type        = string
  default     = null
}

variable "instance_parameter_mapping" {
  description = "Map of instance params to configure."
  type        = list(map(string))
  default     = null
}

## RDS Role Attachment

variable "rds_cluster_role_association" {
  description = <<-EOT
    (Optional) Manages an RDS DB Cluster association with an IAM Role.
    Example use cases:
      - Creating an IAM Role to Allow Amazon Aurora to Access AWS Services
      - Importing/Exporting between RDS PostgreSQL DB Cluster and Amazon S3
        - see https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/postgresql-s3-export.html#postgresql-s3-export-access-bucket

    "featureName" is an Enum and is specific to the engine_version being utilized.
      This can be found in the AWS documentation relevant to the integration or a full
        list is available in the SupportedFeatureNames list returned by AWS CLI rds
        describe-db-engine-versions.
      Example:
        aws rds describe-db-engine-versions --output table --engine aurora-postgresql \
          --query 'DBEngineVersions[].{ a_Version: EngineVersion, b_Features: SupportedFeatureNames.join(`,`,@)}'

    At the time of this update, enginge versions >=11 support the following features:
      - Comprehend
      - Lambda
      - s3Export
      - s3Import
      - SageMaker

    "roleArn" is the IAM role ARN to be applied to the cluster.
    EOT
  type = object({
    featureName = string
    roleArn     = string
  })
  default = null
}
